@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.129 2006/04/19 09:48:02 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.135 2006/07/26 13:33:13 lav Exp $ */
 
 #include <config.h>
 
@@ -41,6 +41,7 @@
 #include "ArgV.h"
 #include "ConnectionSlot.h"
 #include "SignalHook.h"
+#include "FileGlob.h"
 #ifdef WITH_MODULES
 # include "module.h"
 #endif
@@ -64,12 +65,12 @@ void FileAccess::Init()
    default_cwd="~";
    cwd.Set(default_cwd,false,0);
    new_cwd=0;
-   real_pos=-1;
+   limit=FILE_END;
+   real_pos=UNKNOWN_POS;
    pos=0;
    mode=CLOSED;
    try_time=0;
    retries=0;
-   event_time=now;
    opt_date=0;
    opt_size=0;
    error=0;
@@ -84,8 +85,6 @@ void FileAccess::Init()
    entity_charset=0;
    array_ptr=array_cnt=0;
 
-   url=0;
-
    entity_size=NO_SIZE;
    entity_date=NO_DATE;
 
@@ -129,7 +128,6 @@ FileAccess::~FileAccess()
    xfree(pass);
    xfree(hostname);
    xfree(portname);
-   xfree(url);
    xfree(closure);
    xfree(location);
    xfree(entity_content_type);
@@ -158,59 +156,6 @@ void  FileAccess::DebugPrint(const char *prefix,const char *str,int level)
    Log::global->Write(level,buf);
 }
 
-int   FileAccess::Poll(int fd,int ev)
-{
-   struct pollfd pfd;
-   pfd.fd=fd;
-   pfd.events=ev;
-   pfd.revents=0;
-   int res=poll(&pfd,1,0);
-   if(res<1)
-      return 0;
-   if(CheckHangup(&pfd,1))
-      return -1;
-   if(pfd.revents)
-      event_time=now;
-   return pfd.revents;
-}
-
-int   FileAccess::CheckHangup(struct pollfd *pfd,int num)
-{
-   for(int i=0; i<num; i++)
-   {
-#ifdef SO_ERROR
-      char  str[256];
-      int   s_errno=0;
-      socklen_t len;
-
-      errno=0;
-
-// Where does the error number go - to errno or to the pointer?
-// It seems that to errno, but if the pointer is NULL it dumps core.
-// (solaris 2.5)
-// It seems to be different on glibc 2.0 - check both errno and s_errno
-
-      len=sizeof(s_errno);
-      getsockopt(pfd[i].fd,SOL_SOCKET,SO_ERROR,(char*)&s_errno,&len);
-      if(errno==ENOTSOCK)
-	 return 0;
-      if(errno!=0 || s_errno!=0)
-      {
-	 sprintf(str,_("Socket error (%s) - reconnecting"),
-				    strerror(errno?errno:s_errno));
-	 DebugPrint("**** ",str,0);
-	 return 1;
-      }
-#endif /* SO_ERROR */
-      if(pfd[i].revents&POLLERR)
-      {
-	 DebugPrint("**** ","POLLERR",0);
-	 return 1;
-      }
-   } /* end for */
-   return 0;
-}
-
 void FileAccess::NonBlock(int fd)
 {
    int fl=fcntl(fd,F_GETFL);
@@ -230,9 +175,8 @@ void  FileAccess::Open(const char *fn,int mode,off_t offs)
    if(IsOpen())
       Close();
    Resume();
-   event_time=now;
    file=xstrdup(fn);
-   real_pos=-1;
+   real_pos=UNKNOWN_POS;
    pos=offs;
    this->mode=mode;
    mkdir_p=false;
@@ -244,11 +188,11 @@ void  FileAccess::Open(const char *fn,int mode,off_t offs)
    case REMOVE:
    case MAKE_DIR:
    case CHANGE_MODE:
-      LsCache::FileChanged(this,file);
+      cache->FileChanged(this,file);
       break;
    case REMOVE_DIR:
-      LsCache::FileChanged(this,file);
-      LsCache::TreeChanged(this,file);
+      cache->FileChanged(this,file);
+      cache->TreeChanged(this,file);
       break;
    default:
       break;
@@ -362,9 +306,9 @@ void FileAccess::Rename(const char *f,const char *f1)
    file1=xstrdup(f1);
    Open(f,RENAME);
 
-   LsCache::TreeChanged(this,file);
-   LsCache::FileChanged(this,file);
-   LsCache::FileChanged(this,file1);
+   cache->TreeChanged(this,file);
+   cache->FileChanged(this,file);
+   cache->FileChanged(this,file1);
 }
 
 void FileAccess::Mkdir(const char *fn,bool allp)
@@ -373,19 +317,19 @@ void FileAccess::Mkdir(const char *fn,bool allp)
    mkdir_p=allp;
 }
 
-bool FileAccess::SameLocationAs(FileAccess *fa)
+bool FileAccess::SameLocationAs(const FileAccess *fa) const
 {
    return SameSiteAs(fa);
 }
-bool FileAccess::SameSiteAs(FileAccess *fa)
+bool FileAccess::SameSiteAs(const FileAccess *fa) const
 {
-   if(strcmp(this->GetProto(),fa->GetProto()))
-      return false;
-   return true;
+   return SameProtoAs(fa);
 }
 
-const char *FileAccess::GetFileURL(const char *f,int flags)
+const char *FileAccess::GetFileURL(const char *f,int flags) const
 {
+   static char *url;
+
    const char *proto=GetVisualProto();
    if(proto[0]==0)
       return "";
@@ -425,7 +369,7 @@ const char *FileAccess::GetFileURL(const char *f,int flags)
    return url;
 }
 
-const char *FileAccess::GetConnectURL(int flags)
+const char *FileAccess::GetConnectURL(int flags) const
 {
    return GetFileURL(0,flags);
 }
@@ -883,17 +827,12 @@ bool FileAccess::NotSerious(int e)
    return false;
 }
 
-void FileAccess::BumpEventTime(time_t t)
-{
-   if(event_time<t)
-      event_time=t;
-}
 void FileAccess::SetTryTime(time_t t)
 {
    try_time=t;
 }
 
-ResValue FileAccess::Query(const char *name,const char *closure)
+ResValue FileAccess::Query(const char *name,const char *closure) const
 {
    const char *prefix=res_prefix;
    if(!prefix)
@@ -903,24 +842,24 @@ ResValue FileAccess::Query(const char *name,const char *closure)
    return ResMgr::Query(fullname,closure);
 }
 
-bool FileAccess::IsBetterThan(FileAccess *fa)
+bool FileAccess::IsBetterThan(const FileAccess *fa) const
 {
    return(SameProtoAs(fa) && this->IsConnected() > fa->IsConnected());
 }
 
 void FileAccess::Reconfig(const char *) {}
-void FileAccess::ConnectVerify() {}
+void FileAccess::ConnectVerify() { mode=CONNECT_VERIFY; }
 const char *FileAccess::CurrentStatus() { return ""; }
 int FileAccess::Buffered() { return 0; }
 bool FileAccess::IOReady() { return IsOpen(); }
-int FileAccess::IsConnected() { return 0; }
+int FileAccess::IsConnected() const { return 0; }
 void FileAccess::Disconnect() {}
 void FileAccess::UseCache(bool) {}
 bool FileAccess::NeedSizeDateBeforehand() { return false; }
 void FileAccess::Cleanup() {}
 void FileAccess::CleanupThis() {}
 ListInfo *FileAccess::MakeListInfo(const char *path) { return 0; }
-Glob *FileAccess::MakeGlob(const char *pattern) { return 0; }
+Glob *FileAccess::MakeGlob(const char *pattern) { return new NoGlob(pattern); }
 DirList *FileAccess::MakeDirList(ArgV *a) { if(a) delete a; return 0; }
 
 DirList::~DirList()
@@ -1238,6 +1177,9 @@ void FileAccess::Path::ExpandTilde(const Path &home)
    expand_tilde(&path,home.path);
 }
 
+#include "DirColors.h"
+#include "LocalDir.h"
+#include "FileCopy.h"
 #include "modconfig.h"
 #ifndef MODULE_PROTO_FTP
 # include "ftpclass.h"
@@ -1270,11 +1212,13 @@ void FileAccess::Path::ExpandTilde(const Path &home)
 # define _sftp
 #endif
 bool FileAccess::class_inited;
+LsCache *FileAccess::cache;
 void FileAccess::ClassInit()
 {
    if(class_inited)
       return;
    class_inited=true;
+   cache=new LsCache();
 
    SignalHook::ClassInit();
    ResMgr::ClassInit();
@@ -1284,4 +1228,14 @@ void FileAccess::ClassInit()
    _http;
    _fish;
    _sftp;
+
+   // make it link in classes required by modules.
+   LocalDirectory ld;
+}
+void FileAccess::ClassCleanup()
+{
+   DirColors::DeleteInstance();
+   delete cache;
+   cache=0;
+   FileCopy::fxp_create=0;
 }
