@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.43 2004/11/13 12:28:00 lav Exp $ */
+/* $Id: FileSet.cc,v 1.51 2006/07/27 07:44:53 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -73,8 +73,10 @@ void FileInfo::SetName(const char *n)
 {
    if(n==name)
       return;
+   // in case of n being tail of name, dup it first
+   char *n1=xstrdup(n);
    xfree(name);
-   name=xstrdup(n);
+   name=n1;
    defined|=NAME;
 }
 
@@ -94,6 +96,17 @@ void FileInfo::SetGroup(const char *g)
    defined|=GROUP;
 }
 
+void FileSet::add_before(int pos,FileInfo *fi)
+{
+   if(fnum==allocated)
+   {
+      allocated=allocated?allocated*2:16;
+      files=files_sort=(FileInfo**)xrealloc(files,allocated*sizeof(*files));
+   }
+   memmove(files+pos+1, files+pos, sizeof(*files)*(fnum-pos));
+   files[pos]=fi;
+   fnum++;
+}
 void FileSet::Add(FileInfo *fi)
 {
    assert(!sorted);
@@ -109,9 +122,7 @@ void FileSet::Add(FileInfo *fi)
       delete fi;
       return;
    }
-   files=files_sort=(FileInfo**)xrealloc(files,(++fnum)*sizeof(*files));
-   memmove(files+pos+1, files+pos, sizeof(*files)*(fnum-pos-1));
-   files[pos]=fi;
+   add_before(pos,fi);
 }
 
 void FileSet::Sub(int i)
@@ -127,21 +138,14 @@ void FileSet::Sub(int i)
 
 void FileSet::Merge(const FileSet *set)
 {
-   int i,j;
-   for(i=0; i<set->fnum; i++)
+   for(int i=0; i<set->fnum; i++)
    {
-      for(j=0; j<fnum; j++)
-      {
-      	 if(!strcmp(files[j]->name,set->files[i]->name))
-	 {
-	    files[j]->Merge(*(set->files[i]));
-	    break;
-	 }
-      }
-      if(j==fnum)
-      {
-	 Add(new FileInfo(*set->files[i]));
-      }
+      FileInfo *fi=set->files[i];
+      int pos = FindGEIndByName(fi->name);
+      if(pos < fnum && !strcmp(files[pos]->name,fi->name))
+	 files[pos]->Merge(*fi);
+      else
+	 add_before(pos,new FileInfo(*fi));
    }
 }
 
@@ -161,26 +165,27 @@ FileSet::FileSet(FileSet const *set)
    if(fnum==0)
       files=files_sort=0;
    else
-      files=files_sort=(FileInfo**)xmalloc(fnum*sizeof(*files));
+      files=files_sort=(FileInfo**)xmalloc((allocated=fnum)*sizeof(*files));
    for(int i=0; i<fnum; i++)
       files[i]=new FileInfo(*(set->files[i]));
 }
 
 static int (*compare)(const char *s1, const char *s2);
+static int rev_cmp;
 
 static int sort_name(const void *s1, const void *s2)
 {
    const FileInfo *p1 = *(const FileInfo **) s1;
    const FileInfo *p2 = *(const FileInfo **) s2;
-   return compare(p1->name, p2->name);
+   return compare(p1->name, p2->name) * rev_cmp;
 }
 
 static int sort_size(const void *s1, const void *s2)
 {
    const FileInfo *p1 = *(const FileInfo **) s1;
    const FileInfo *p2 = *(const FileInfo **) s2;
-   if(p1->size > p2->size) return -1;
-   if(p1->size < p2->size) return 1;
+   if(p1->size > p2->size) return -rev_cmp;
+   if(p1->size < p2->size) return rev_cmp;
    return 0;
 }
 
@@ -188,8 +193,8 @@ static int sort_dirs(const void *s1, const void *s2)
 {
    const FileInfo *p1 = *(const FileInfo **) s1;
    const FileInfo *p2 = *(const FileInfo **) s2;
-   if(p1->filetype == FileInfo::DIRECTORY && !p2->filetype == FileInfo::DIRECTORY) return -1;
-   if(!p1->filetype == FileInfo::DIRECTORY && p2->filetype == FileInfo::DIRECTORY) return 1;
+   if(p1->filetype == FileInfo::DIRECTORY && !p2->filetype == FileInfo::DIRECTORY) return -rev_cmp;
+   if(!p1->filetype == FileInfo::DIRECTORY && p2->filetype == FileInfo::DIRECTORY) return rev_cmp;
    return 0;
 }
 
@@ -199,7 +204,7 @@ static int sort_rank(const void *s1, const void *s2)
    const FileInfo *p2 = *(const FileInfo **) s2;
    if(p1->GetRank()==p2->GetRank())
       return sort_name(s1,s2);
-   return p1->GetRank()<p2->GetRank() ? -1 : 1;
+   return p1->GetRank()<p2->GetRank() ? -rev_cmp : rev_cmp;
 }
 
 static int sort_date(const void *s1, const void *s2)
@@ -208,15 +213,15 @@ static int sort_date(const void *s1, const void *s2)
    const FileInfo *p2 = *(const FileInfo **) s2;
    if(p1->date==p2->date)
       return sort_name(s1,s2);
-   return p1->date>p2->date ? -1 : 1;
+   return p1->date>p2->date ? -rev_cmp : rev_cmp;
 }
 
 /* files_sort is an alias of files when sort == NAME (since
  * files is always sorted by name), and an independant array
  * of pointers (pointing to the same data) otherwise. */
-void FileSet::Sort(sort_e newsort, bool casefold)
+void FileSet::Sort(sort_e newsort, bool casefold, bool reverse)
 {
-   if(newsort == BYNAME && !casefold) {
+   if(newsort == BYNAME && !casefold && !reverse) {
       Unsort();
       return;
    }
@@ -232,6 +237,8 @@ void FileSet::Sort(sort_e newsort, bool casefold)
    if(casefold) compare = strcasecmp;
    else compare = strcmp;
 
+   rev_cmp=(reverse?-1:1);
+
    switch(newsort) {
    case BYNAME: qsort(files_sort, fnum, sizeof(FileInfo *), sort_name); break;
    case BYSIZE: qsort(files_sort, fnum, sizeof(FileInfo *), sort_size); break;
@@ -259,6 +266,7 @@ void FileSet::Empty()
    xfree(files);
    files=0; files_sort=0;
    fnum=0;
+   allocated=0;
    ind=0;
 }
 
@@ -315,6 +323,34 @@ void FileSet::SubtractOlderThan(time_t t)
       }
    }
 }
+void FileSet::SubtractNewerThan(time_t t)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype!=FileInfo::NORMAL)
+	 continue;
+      if(files[i]->NewerThan(t))
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
+void FileSet::SubtractSizeOutside(const Range *r)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype!=FileInfo::NORMAL)
+	 continue;
+      if(files[i]->SizeOutside(r))
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
 void FileSet::SubtractDirs()
 {
    for(int i=0; i<fnum; i++)
@@ -391,7 +427,15 @@ bool  FileInfo::SameAs(const FileInfo *fi,int ignore)
 
 bool  FileInfo::OlderThan(time_t t)
 {
-   return ((defined&DATE) && date<t);
+   return((defined&DATE) && date<t);
+}
+bool  FileInfo::NewerThan(time_t t)
+{
+   return((defined&DATE) && date>=t);
+}
+bool  FileInfo::SizeOutside(const Range *r)
+{
+   return((defined&SIZE) && !r->Match(size));
 }
 
 void FileSet::Count(int *d,int *f,int *s,int *o)
@@ -592,7 +636,10 @@ void FileSet::LocalChown(const char *dir)
 	       new_gid=g;
 	 }
 	 if(new_uid!=st.st_uid || new_gid!=st.st_gid)
-	    lchown(local_name,new_uid,new_gid);
+	 {
+	    if(lchown(local_name,new_uid,new_gid)==-1)
+	       /* don't care */;
+	 }
       }
    }
 }
@@ -820,7 +867,8 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    int prec=30;
    if(strlen(t)==5)
    {
-      sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min);
+      if(sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min)!=2)
+	 ERR;
       date.tm_year=guess_year(date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min) - 1900;
    }
    else
@@ -948,7 +996,7 @@ void FileInfo::MakeLongName()
 
 int FileSet::EstimateMemory() const
 {
-   int size=sizeof(FileSet)+sizeof(FileInfo*)*fnum;
+   int size=sizeof(FileSet)+sizeof(FileInfo*)*allocated;
    for(int i=0; i<fnum; i++)
    {
       size+=sizeof(FileInfo);
