@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.208 2006/05/13 05:48:33 lav Exp $ */
+/* $Id: Http.cc,v 1.216 2006/07/26 13:15:26 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -141,7 +141,7 @@ void Http::MoveConnectionHere(Http *o)
    sock=o->sock; o->sock=-1;
    rate_limit=o->rate_limit; o->rate_limit=0;
    last_method=o->last_method; o->last_method=0;
-   BumpEventTime(o->event_time);
+   timeout_timer.Reset(o->timeout_timer);
    state=CONNECTED;
    o->Disconnect();
    ResumeInternal();
@@ -577,7 +577,12 @@ void Http::SendRequest(const char *connection,const char *f)
    retrieve:
       SendMethod("GET",efile);
       if(pos>0 && !no_ranges)
-	 Send("Range: bytes=%lld-\r\n",(long long)pos);
+      {
+	 if(limit==FILE_END)
+	    Send("Range: bytes=%lld-\r\n",(long long)pos);
+	 else
+	    Send("Range: bytes=%lld-%lld\r\n",(long long)pos,(long long)limit-1);
+      }
       break;
 
    case STORE:
@@ -592,10 +597,18 @@ void Http::SendRequest(const char *connection,const char *f)
       if(entity_size>=0)
 	 Send("Content-length: %lld\r\n",(long long)(entity_size-pos));
       if(pos>0 && entity_size<0)
-	 Send("Range: bytes=%lld-\r\n",(long long)pos);
+      {
+	 if(limit==FILE_END)
+	    Send("Range: bytes=%lld-\r\n",(long long)pos);
+	 else
+	    Send("Range: bytes=%lld-%lld\r\n",(long long)pos,(long long)limit-1);
+      }
       else if(pos>0)
+      {
 	 Send("Range: bytes=%lld-%lld/%lld\r\n",(long long)pos,
-		     (long long)(entity_size-1),(long long)entity_size);
+		     (long long)((limit==FILE_END || limit>entity_size ? entity_size : limit)-1),
+		     (long long)entity_size);
+      }
       if(entity_date!=NO_DATE)
       {
 	 char d[256];
@@ -946,15 +959,11 @@ int Http::Do()
    int len;
 
    // check if idle time exceeded
-   if(mode==CLOSED && sock!=-1 && idle>0)
+   if(mode==CLOSED && sock!=-1 && idle_timer.Stopped())
    {
-      if(now >= idle_start+idle)
-      {
-	 DebugPrint("---- ",_("Closing idle connection"),1);
-	 Disconnect();
-	 return m;
-      }
-      TimeoutS(idle_start+idle-time_t(now));
+      DebugPrint("---- ",_("Closing idle connection"),1);
+      Disconnect();
+      return m;
    }
 
    if(home.path==0)
@@ -1092,8 +1101,8 @@ int Http::Do()
 	 return MOVED;
       }
       KeepAlive(sock);
-      SetSocketBuffer(sock,socket_buffer);
-      SetSocketMaxseg(sock,socket_maxseg);
+      SetSocketBuffer(sock);
+      SetSocketMaxseg(sock);
       NonBlock(sock);
       CloseOnExec(sock);
 
@@ -1114,7 +1123,7 @@ int Http::Do()
       }
       state=CONNECTING;
       m=MOVED;
-      event_time=now;
+      timeout_timer.Reset();
 
    case CONNECTING:
       res=Poll(sock,POLLOUT);
@@ -1229,8 +1238,8 @@ int Http::Do()
 	 Disconnect();
 	 return MOVED;
       }
-      BumpEventTime(send_buf->EventTime());
-      BumpEventTime(recv_buf->EventTime());
+      timeout_timer.Reset(send_buf->EventTime());
+      timeout_timer.Reset(recv_buf->EventTime());
       if(CheckTimeout())
 	 return MOVED;
       recv_buf->Get(&buf,&len);
@@ -1569,8 +1578,8 @@ int Http::Do()
 	    if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
 	       m=MOVED;
 	 }
-	 BumpEventTime(send_buf->EventTime());
-	 BumpEventTime(recv_buf->EventTime());
+	 timeout_timer.Reset(send_buf->EventTime());
+	 timeout_timer.Reset(recv_buf->EventTime());
 	 if(recv_buf->Size()==0)
 	 {
 	    // check if ranges were emulated by squid
@@ -1930,14 +1939,15 @@ void Http::Reconfig(const char *name)
    }
 
    if(sock!=-1)
-      SetSocketBuffer(sock,socket_buffer);
+      SetSocketBuffer(sock);
    if(proxy && proxy_port==0)
       proxy_port=xstrdup(HTTP_DEFAULT_PROXY_PORT);
 
    user_agent=ResMgr::Query("http:user-agent",c);
+   use_propfind_now=(use_propfind_now && QueryBool("use-propfind",c));
 }
 
-bool Http::SameSiteAs(FileAccess *fa)
+bool Http::SameSiteAs(const FileAccess *fa) const
 {
    if(!SameProtoAs(fa))
       return false;
@@ -1946,7 +1956,7 @@ bool Http::SameSiteAs(FileAccess *fa)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass));
 }
 
-bool Http::SameLocationAs(FileAccess *fa)
+bool Http::SameLocationAs(const FileAccess *fa) const
 {
    if(!SameSiteAs(fa))
       return false;
@@ -1961,6 +1971,7 @@ void Http::ResetLocationData()
    super::ResetLocationData();
    Reconfig();
    state=DISCONNECTED;
+   use_propfind_now=QueryBool("use-propfind",hostname);
 }
 
 DirList *Http::MakeDirList(ArgV *args)
@@ -2199,6 +2210,7 @@ void Http::MakeSSLBuffers()
    Delete(recv_buf);
 
    lftp_ssl *ssl=new lftp_ssl(sock,lftp_ssl::CLIENT,hostname);
+   ssl->load_keys();
    IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
    IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
    recv_buf_ssl->CloseLater();
