@@ -18,12 +18,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.h,v 1.13 2003/12/01 07:46:21 lav Exp $ */
+/* $Id: HttpDir.h,v 1.19 2005/06/22 08:04:20 lav Exp $ */
 
 #ifndef HTTPDIR_H
 #define HTTPDIR_H
 
 #include "Http.h"
+#if USE_EXPAT
+# include <expat.h>
+#endif
 
 class HttpListInfo : public GenericParseListInfo
 {
@@ -34,6 +37,7 @@ public:
       {
 	 get_time_for_dirs=false;
       }
+   static FileSet *ParseProps(const char *buf,int len,const char *base_dir);
 };
 
 class ParsedURL;
@@ -46,18 +50,26 @@ class HttpDirList : public DirList
    ParsedURL *curr_url;
    FileSet all_links;
    int mode;
+   bool parse_as_html;
    char *base_href;
 
+#if USE_EXPAT
+   XML_Parser xml_p;
+   struct xml_context *xml_ctx;
+#endif
+
    LsOptions ls_options;
 
+   void ParsePropsFormat(const char *b,int len,bool eof);
+
 public:
    HttpDirList(ArgV *a,FileAccess *fa);
    ~HttpDirList();
    const char *Status();
    int Do();
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 };
 
 #endif//HTTPDIR_H
