@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.50 2005/06/28 08:55:17 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.51 2006/06/13 14:35:38 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -529,7 +529,7 @@ const char *LocalAccess::CurrentStatus()
    return "";
 }
 
-bool LocalAccess::SameLocationAs(FileAccess *fa)
+bool LocalAccess::SameLocationAs(const FileAccess *fa) const
 {
    if(!SameProtoAs(fa))
       return false;
