@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.134 2006/06/30 14:10:28 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.138 2006/08/08 05:41:54 lav Exp $ */
 
 #include <config.h>
 
@@ -151,7 +151,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
       if(target_list_info && (!source_list_info || now%4>=2))
       {
 	 const char *status=target_list_info->Status();
-	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int status_w=mbswidth(status, 0);
 	 int dw=w-status_w;
 	 if(dw<20)
 	    dw=20;
@@ -163,7 +163,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
       else if(source_list_info)
       {
 	 const char *status=source_list_info->Status();
-	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int status_w=mbswidth(status, 0);
 	 int dw=w-status_w;
 	 if(dw<20)
 	    dw=20;
@@ -297,7 +297,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       c->RemoveTargetFirst();
 	    CopyJob *cp=
 	       new CopyJob(c,file->name,"mirror");
-	    if(file->defined&file->DATE)
+	    if((file->defined&file->DATE) && file->date_prec<=1)
 	       cp->SetDate(file->date);
 	    if(file->defined&file->SIZE)
 	       cp->SetSize(file->size);
@@ -1261,11 +1261,31 @@ CMD(mirror)
 {
 #define args (parent->args)
 #define eprintf parent->eprintf
-   static struct option mirror_opts[]=
+   enum {
+      OPT_ALLOW_CHOWN,
+      OPT_DELETE_FIRST,
+      OPT_IGNORE_SIZE,
+      OPT_IGNORE_TIME,
+      OPT_LOOP,
+      OPT_MAX_ERRORS,
+      OPT_NO_DEREFERENCE,
+      OPT_NO_SYMLINKS,
+      OPT_NO_UMASK,
+      OPT_OLDER_THAN,
+      OPT_ONLY_MISSING,
+      OPT_PERMS,
+      OPT_REMOVE_SOURCE_FILES,
+      OPT_SCRIPT,
+      OPT_SCRIPT_ONLY,
+      OPT_SIZE_RANGE,
+      OPT_USE_CACHE,
+      OPT_USE_PGET_N,
+   };
+   static const struct option mirror_opts[]=
    {
       {"delete",no_argument,0,'e'},
       {"allow-suid",no_argument,0,'s'},
-      {"allow-chown",no_argument,0,256+'a'},
+      {"allow-chown",no_argument,0,OPT_ALLOW_CHOWN},
       {"include",required_argument,0,'i'},
       {"exclude",required_argument,0,'x'},
       {"include-glob",required_argument,0,'I'},
@@ -1273,31 +1293,31 @@ CMD(mirror)
       {"only-newer",no_argument,0,'n'},
       {"no-recursion",no_argument,0,'r'},
       {"no-perms",no_argument,0,'p'},
-      {"perms",no_argument,0,'p'+512},
-      {"no-umask",no_argument,0,256+'u'},
+      {"perms",no_argument,0,OPT_PERMS},
+      {"no-umask",no_argument,0,OPT_NO_UMASK},
       {"continue",no_argument,0,'c'},
       {"reverse",no_argument,0,'R'},
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
-      {"older-than",required_argument,0,256+'O'},
-      {"size-range",required_argument,0,512+'S'},
+      {"older-than",required_argument,0,OPT_OLDER_THAN},
+      {"size-range",required_argument,0,OPT_SIZE_RANGE},
       {"dereference",no_argument,0,'L'},
-      {"no-dereference",no_argument,0,'L'+512},
-      {"use-cache",no_argument,0,256+'C'},
-      {"Remove-source-files",no_argument,0,256+'R'},
+      {"no-dereference",no_argument,0,OPT_NO_DEREFERENCE},
+      {"use-cache",no_argument,0,OPT_USE_CACHE},
+      {"Remove-source-files",no_argument,0,OPT_REMOVE_SOURCE_FILES},
       {"parallel",optional_argument,0,'P'},
-      {"ignore-time",no_argument,0,256+'i'},
-      {"ignore-size",no_argument,0,257+'i'},
-      {"only-missing",no_argument,0,256+'m'},
-      {"log",required_argument,0,256+'s'},
-      {"script",    required_argument,0,256+'S'},
-      {"just-print",optional_argument,0,256+'S'},
-      {"dry-run",   optional_argument,0,256+'S'},
-      {"delete-first",no_argument,0,256+'e'},
-      {"use-pget-n",optional_argument,0,256+'p'},
-      {"no-symlinks",no_argument,0,256+'y'},
-      {"loop",no_argument,0,256+'l'},
-      {"max-errors",required_argument,0,256+'E'},
+      {"ignore-time",no_argument,0,OPT_IGNORE_TIME},
+      {"ignore-size",no_argument,0,OPT_IGNORE_SIZE},
+      {"only-missing",no_argument,0,OPT_ONLY_MISSING},
+      {"log",required_argument,0,OPT_SCRIPT},
+      {"script",    required_argument,0,OPT_SCRIPT_ONLY},
+      {"just-print",optional_argument,0,OPT_SCRIPT_ONLY},
+      {"dry-run",   optional_argument,0,OPT_SCRIPT_ONLY},
+      {"delete-first",no_argument,0,OPT_DELETE_FIRST},
+      {"use-pget-n",optional_argument,0,OPT_USE_PGET_N},
+      {"no-symlinks",no_argument,0,OPT_NO_SYMLINKS},
+      {"loop",no_argument,0,OPT_LOOP},
+      {"max-errors",required_argument,0,OPT_MAX_ERRORS},
       {0}
    };
 
@@ -1323,6 +1343,7 @@ CMD(mirror)
 
    PatternSet *exclude=0;
    const char *default_exclude=ResMgr::Query("mirror:exclude-regex",0);
+   const char *default_include=ResMgr::Query("mirror:include-regex",0);
 
    if(!ResMgr::QueryBool("mirror:set-permissions",0))
       flags|=MirrorJob::NO_PERMS;
@@ -1340,7 +1361,7 @@ CMD(mirror)
       case('s'):
 	 flags|=MirrorJob::ALLOW_SUID;
 	 break;
-      case(256+'a'):
+      case(OPT_ALLOW_CHOWN):
 	 flags|=MirrorJob::ALLOW_CHOWN;
 	 break;
       case('a'):
@@ -1355,7 +1376,7 @@ CMD(mirror)
       case('p'):
 	 flags|=MirrorJob::NO_PERMS;
 	 break;
-      case('p'+512):
+      case(OPT_PERMS):
 	 flags&=~MirrorJob::NO_PERMS;
 	 break;
       case('c'):
@@ -1393,7 +1414,11 @@ CMD(mirror)
 	     * explicit pattern is for exclusion - otherwise all files are
 	     * excluded by default and no default exclusion is needed. */
 	    if(type==PatternSet::EXCLUDE && default_exclude && *default_exclude)
+	    {
 	       exclude->Add(type,new PatternSet::Regex(default_exclude));
+	       if(default_include && *default_include)
+		  exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
+	    }
 	    default_exclude=0;
 	    /* Users usually don't want to exclude all directories */
 	    if(type==PatternSet::INCLUDE)
@@ -1408,7 +1433,7 @@ CMD(mirror)
       case('L'):
 	 flags|=MirrorJob::RETR_SYMLINKS;
 	 break;
-      case('L'+512):
+      case(OPT_NO_DEREFERENCE):
 	 flags&=~MirrorJob::RETR_SYMLINKS;
 	 break;
       case('v'):
@@ -1422,10 +1447,10 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
-      case(256+'O'):
+      case(OPT_OLDER_THAN):
 	 older_than=optarg;
 	 break;
-      case(512+'S'):
+      case(OPT_SIZE_RANGE):
 	 size_range=new Range(optarg);
 	 if(size_range->Error())
 	 {
@@ -1434,22 +1459,22 @@ CMD(mirror)
 	    goto no_job;
 	 }
 	 break;
-      case(256+'u'):
+      case(OPT_NO_UMASK):
 	 flags|=MirrorJob::NO_UMASK;
 	 break;
-      case(256+'C'):
+      case(OPT_USE_CACHE):
 	 use_cache=true;
 	 break;
-      case(256+'R'):
+      case(OPT_REMOVE_SOURCE_FILES):
 	 remove_source_files=true;
 	 break;
-      case(256+'i'):
+      case(OPT_IGNORE_TIME):
 	 flags|=MirrorJob::IGNORE_TIME;
 	 break;
-      case(257+'i'):
+      case(OPT_IGNORE_SIZE):
 	 flags|=MirrorJob::IGNORE_SIZE;
 	 break;
-      case(256+'m'):
+      case(OPT_ONLY_MISSING):
 	 flags|=MirrorJob::IGNORE_TIME|MirrorJob::IGNORE_SIZE;
 	 break;
       case('P'):
@@ -1458,29 +1483,29 @@ CMD(mirror)
 	 else
 	    parallel=3;
 	 break;
-      case(256+'p'):
+      case(OPT_USE_PGET_N):
 	 if(optarg)
 	    use_pget=atoi(optarg);
 	 else
 	    use_pget=3;
 	 break;
-      case(256+'S'):
+      case(OPT_SCRIPT_ONLY):
 	 script_only=true;
-      case(256+'s'):
+      case(OPT_SCRIPT):
 	 script_file=optarg;
 	 if(script_file==0)
 	    script_file="-";
 	 break;
-      case(256+'e'):
+      case(OPT_DELETE_FIRST):
 	 flags|=MirrorJob::REMOVE_FIRST|MirrorJob::DELETE;
 	 break;
-      case(256+'y'):
+      case(OPT_NO_SYMLINKS):
 	 flags|=MirrorJob::NO_SYMLINKS;
 	 break;
-      case(256+'l'):
+      case(OPT_LOOP):
 	 flags|=MirrorJob::LOOP;
 	 break;
-      case(256+'E'):
+      case(OPT_MAX_ERRORS):
 	 max_error_count=atoi(optarg);
 	 break;
       case('?'):
@@ -1501,6 +1526,8 @@ CMD(mirror)
    {
       exclude=new PatternSet;
       exclude->Add(PatternSet::EXCLUDE,new PatternSet::Regex(default_exclude));
+      if(default_include && *default_include)
+	 exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
    }
 
    args->back();
