@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ProcWait.cc,v 1.16 2004/01/14 08:23:46 lav Exp $ */
+/* $Id: ProcWait.cc,v 1.18 2006/02/06 11:04:00 lav Exp $ */
 
 #include <config.h>
 #include <sys/wait.h>
@@ -159,6 +159,6 @@ void ProcWait::Signal(bool yes)
 void ProcWait::DeleteAll()
 {
    Signal(false);
-   while(chain)
-      Delete(chain);
+   for(ProcWait *scan=chain; scan; scan=scan->next)
+      scan->deleting=true;
 }
