@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.59 2006/02/06 10:57:27 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.64 2006/06/30 14:25:11 lav Exp $ */
 
 #include <config.h>
 
@@ -41,7 +41,7 @@ CDECL_END
 #include "misc.h"
 
 ResMgr::Resource  *ResMgr::chain=0;
-ResDecl		  *ResMgr::type_chain=0;
+ResType		  *ResMgr::type_chain=0;
 
 int ResMgr::VarNameCmp(const char *good_name,const char *name)
 {
@@ -78,15 +78,15 @@ int ResMgr::VarNameCmp(const char *good_name,const char *name)
    return res;
 }
 
-const char *ResMgr::FindVar(const char *name,ResDecl **type)
+const char *ResMgr::FindVar(const char *name,const ResType **type)
 {
-   ResDecl *exact_proto=0;
-   ResDecl *exact_name=0;
+   const ResType *exact_proto=0;
+   const ResType *exact_name=0;
 
    *type=0;
 
    int sub=0;
-   ResDecl *type_scan;
+   const ResType *type_scan;
    for(type_scan=type_chain; type_scan; type_scan=type_scan->next)
    {
       switch(VarNameCmp(type_scan->name,name))
@@ -124,11 +124,20 @@ const char *ResMgr::FindVar(const char *name,ResDecl **type)
    return _("ambiguous variable name");
 }
 
+const ResType *ResMgr::FindRes(const char *name)
+{
+   const ResType *type;
+   const char *msg=FindVar(name,&type);
+   if(msg)
+      return 0;
+   return type;
+}
+
 const char *ResMgr::Set(const char *name,const char *cclosure,const char *cvalue)
 {
    const char *msg;
 
-   ResDecl *type;
+   const ResType *type;
    // find type of given variable
    msg=FindVar(name,&type);
    if(msg)
@@ -215,7 +224,7 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
    char *res;
 
    Resource *scan;
-   ResDecl  *dscan;
+   ResType  *dscan;
 
    int n=0;
    int dn=0;
@@ -327,7 +336,7 @@ char **ResMgr::Generator(void)
    char **res;
 
    Resource *scan;
-   ResDecl  *dscan;
+   ResType  *dscan;
 
    int n=0;
    int dn=0;
@@ -498,10 +507,8 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
 
 const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
 {
-   ResDecl *type;
-   // find type of given variable
-   const char *msg=FindVar(name,&type);
-   if(msg)
+   const ResType *type=FindRes(name);
+   if(!type)
       return 0;
 
    if(*ptr==0)
@@ -521,34 +528,28 @@ const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **p
    return 0;
 }
 
+const char *ResMgr::SimpleQuery(const ResType *type,const char *closure)
+{
+   // find the value
+   for(Resource *scan=chain; scan; scan=scan->next)
+      if(scan->type==type && scan->ClosureMatch(closure))
+	 return scan->value;
+   return 0;
+}
 const char *ResMgr::SimpleQuery(const char *name,const char *closure)
 {
-   const char *msg;
-
-   ResDecl *type;
-   // find type of given variable
-   msg=FindVar(name,&type);
-   if(msg)
+   const ResType *type=FindRes(name);
+   if(!type)
       return 0;
 
-   Resource *scan;
-   // find the value
-   for(scan=chain; scan; scan=scan->next)
-      if(scan->type==type && scan->ClosureMatch(closure))
-	 break;
-
-   // if found
-   if(scan)
-      return scan->value;
-
-   return 0;
+   return SimpleQuery(type,closure);
 }
 
 ResValue ResMgr::Query(const char *name,const char *closure)
 {
    const char *msg;
 
-   ResDecl *type;
+   const ResType *type;
    // find type of given variable
    msg=FindVar(name,&type);
    if(msg)
@@ -558,32 +559,17 @@ ResValue ResMgr::Query(const char *name,const char *closure)
       return 0;
    }
 
-   for(;;)
-   {
-      Resource **scan;
-      // find the value
-      for(scan=&chain; *scan; scan=&(*scan)->next)
-	 if((*scan)->type==type && (*scan)->ClosureMatch(closure))
-	    break;
-      // if found
-      if(*scan)
-	 return (*scan)->value;
-      if(!closure)
-	 break;
-      closure=0;
-   }
-
-   return type->defvalue;
+   return type->Query(closure);
 }
 
-ResValue ResDecl::Query(const char *closure)
+ResValue ResType::Query(const char *closure) const
 {
    const char *v=0;
 
    if(closure)
-      v=ResMgr::SimpleQuery(name,closure);
+      v=ResMgr::SimpleQuery(this,closure);
    if(!v)
-      v=ResMgr::SimpleQuery(name,0);
+      v=ResMgr::SimpleQuery(this,0);
    if(!v)
       v=defvalue;
 
@@ -595,19 +581,35 @@ bool ResMgr::str2bool(const char *s)
    return(strchr("TtYy1+",s[0])!=0 || !strcasecmp(s,"on"));
 }
 
-ResDecl::ResDecl(const char *a_name,const char *a_defvalue,
-		  ResValValid *a_val_valid,ResClValid *a_closure_valid)
+ResDecl::ResDecl(const char *a_name,const char *a_defvalue,ResValValid *a_val_valid,ResClValid *a_closure_valid)
 {
    name=a_name;
-   defvalue=xstrdup(a_defvalue);
+   defvalue=a_defvalue;
    val_valid=a_val_valid;
    closure_valid=a_closure_valid;
-   next=ResMgr::type_chain;
-   ResMgr::type_chain=this;
+   ResMgr::AddType(this);
 }
-ResDecl::~ResDecl()
+ResDecls::ResDecls(ResType *array)
 {
-   for(ResDecl **scan=&ResMgr::type_chain; *scan; scan=&(*scan)->next)
+   while(array->name)
+      ResMgr::AddType(array++);
+}
+ResDecls::ResDecls(ResType *r1,ResType *r2,...)
+{
+   ResMgr::AddType(r1);
+   if(!r2)
+      return;
+   ResMgr::AddType(r2);
+   va_list v;
+   va_start(v,r2);
+   while((r1=va_arg(v,ResType *))!=0)
+      ResMgr::AddType(r1);
+   va_end(v);
+}
+
+ResType::~ResType()
+{
+   for(ResType **scan=&ResMgr::type_chain; *scan; scan=&(*scan)->next)
    {
       if(*scan==this)
       {
@@ -624,9 +626,7 @@ ResDecl::~ResDecl()
       {
 	 if((*scan)->type==this)
 	 {
-	    ResMgr::Resource *to_free=*scan;
-	    *scan=(*scan)->next;
-	    delete to_free;
+	    delete replace_value(*scan,(*scan)->next);
 	    modified=true;
 	 }
 	 else
@@ -639,15 +639,9 @@ ResDecl::~ResDecl()
 	 SMTask::ReconfigAll(this->name);
 #endif
    }
-
-   xfree(defvalue);
 }
 
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
-
-void TimeInterval::init(const char *s)
+void TimeIntervalR::init(const char *s)
 {
    double interval=0;
    infty=false;
@@ -694,7 +688,7 @@ void TimeInterval::init(const char *s)
 
 const char *ResMgr::TimeIntervalValidate(char **s)
 {
-   TimeInterval t(*s);
+   TimeIntervalR t(*s);
    if(t.Error())
       return t.ErrorText();
    return 0;
