@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.41 2006/02/06 10:57:31 lav Exp $ */
+/* $Id: misc.h,v 1.42 2006/05/25 07:05:59 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -68,10 +68,6 @@ int percent(off_t offset,off_t size);
 
 #define find_char(buf,len,ch) ((const char *)memchr(buf,ch,len))
 
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
-
 extern const char month_names[][4];
 
 int parse_month(const char *);
